@@ -101,6 +101,7 @@ mod range {
                     max_jitter_ms: 0,
                     include_size_hint: true,
                     max_chunk_size: chunk_size,
+                    ..TestCondowClient::default()
                 };
 
                 for part_size in [1u64, 3, 50, 1_000] {
@@ -121,7 +122,7 @@ mod range {
                                 condow.config.clone(),
                                 IgnoreLocation,
                                 range.clone(),
-                                Default::default(),
+                                (),
                             )
                             .await
                             .unwrap();
@@ -148,6 +149,7 @@ mod range {
                     max_jitter_ms: 0,
                     include_size_hint: true,
                     max_chunk_size: chunk_size,
+                    ..TestCondowClient::default()
                 };
 
                 for part_size in [1u64, 3, 50, 1_000] {
@@ -196,6 +198,7 @@ mod range {
                     max_jitter_ms: 0,
                     include_size_hint: true,
                     max_chunk_size: chunk_size,
+                    ..TestCondowClient::default()
                 };
 
                 for part_size in [1u64, 3, 50, 1_000] {
@@ -216,7 +219,7 @@ mod range {
                                 condow.config.clone(),
                                 IgnoreLocation,
                                 range.clone(),
-                                Default::default(),
+                                (),
                             )
                             .await
                             .unwrap();
@@ -242,6 +245,7 @@ mod range {
                     max_jitter_ms: 0,
                     include_size_hint: true,
                     max_chunk_size: chunk_size,
+                    ..TestCondowClient::default()
                 };
 
                 for part_size in [1u64, 3, 50, 1_000] {
@@ -262,7 +266,7 @@ mod range {
                                 condow.config.clone(),
                                 IgnoreLocation,
                                 range.clone(),
-                                Default::default(),
+                                (),
                             )
                             .await
                             .unwrap();
@@ -296,6 +300,7 @@ mod range {
                             max_jitter_ms: 0,
                             include_size_hint: true,
                             max_chunk_size: chunk_size,
+                            ..TestCondowClient::default()
                         };
 
                         for part_size in [1u64, 3, 50, 1_000] {
@@ -317,7 +322,7 @@ mod range {
                                         condow.config.clone(),
                                         IgnoreLocation,
                                         range,
-                                        Default::default(),
+                                        (),
                                     )
                                     .await
                                     .unwrap();
@@ -343,6 +348,7 @@ mod range {
                             max_jitter_ms: 0,
                             include_size_hint: true,
                             max_chunk_size: chunk_size,
+                            ..TestCondowClient::default()
                         };
 
                         for part_size in [1u64, 3, 50, 1_000] {
@@ -363,7 +369,7 @@ mod range {
                                         condow.config.clone(),
                                         IgnoreLocation,
                                         range,
-                                        Default::default(),
+                                        (),
                                     )
                                     .await
                                     .unwrap();
@@ -397,6 +403,7 @@ mod range {
                             max_jitter_ms: 0,
                             include_size_hint: true,
                             max_chunk_size: chunk_size,
+                            ..TestCondowClient::default()
                         };
 
                         for part_size in [1u64, 33] {
@@ -420,7 +427,7 @@ mod range {
                                             condow.config.clone(),
                                             IgnoreLocation,
                                             range,
-                                            Default::default(),
+                                            (),
                                         )
                                         .await
                                         .unwrap();
@@ -450,6 +457,7 @@ mod range {
                             max_jitter_ms: 0,
                             include_size_hint: true,
                             max_chunk_size: chunk_size,
+                            ..TestCondowClient::default()
                         };
 
                         for part_size in [1u64, 33] {
@@ -473,7 +481,7 @@ mod range {
                                             condow.config.clone(),
                                             IgnoreLocation,
                                             range,
-                                            Default::default(),
+                                            (),
                                         )
                                         .await
                                         .unwrap();
