@@ -23,7 +23,7 @@ pub(crate) async fn download_range<C: CondowClient, DR: Into<DownloadRange>, P:
     config: Config,
     location: C::Location,
     range: DR,
-    probe: ProbeInternal<P>,
+    probe: P,
 ) -> Result<ChunkStream, CondowError> {
     let range = range.into();
 
@@ -99,7 +99,7 @@ async fn download_chunks<C: CondowClient, P: Probe + Clone>(
     range: InclusiveRange,
     bytes_hint: BytesHint,
     config: Config,
-    probe: ProbeInternal<P>,
+    probe: P,
     download_span_guard: DownloadSpanGuard,
 ) -> Result<ChunkStream, CondowError> {
     probe.effective_range(range);
@@ -112,21 +112,14 @@ async fn download_chunks<C: CondowClient, P: Probe + Clone>(
         panic!("n_parts must not be 0. This is a bug");
     }
 
-    let (chunk_stream, sender) = ChunkStream::new(bytes_hint);
-
     let effective_concurrency = config
         .max_concurrency
         .into_inner()
         .min(part_requests.exact_size_hint() as usize);
     if effective_concurrency == 1 {
-        tokio::spawn(download::download_chunks_sequentially(
-            part_requests,
-            client,
-            location,
-            probe,
-            sender,
-        ));
+        download::download_chunks_sequentially(part_requests, client, location, probe).await
     } else {
+        let (chunk_stream, sender) = ChunkStream::new_channel_sink_pair(bytes_hint);
         tokio::spawn(async move {
             let result = download::download_concurrently(
                 part_requests,
@@ -141,9 +134,8 @@ async fn download_chunks<C: CondowClient, P: Probe + Clone>(
             .await;
             result
         });
+        Ok(chunk_stream)
     }
-
-    Ok(chunk_stream)
 }
 
 /// This struct contains a span which must be kept alive until whole download is completed
@@ -165,30 +157,23 @@ impl DownloadSpanGuard {
 /// all the "matches" all over the place...
 #[derive(Clone)]
 pub(crate) enum ProbeInternal<P: Probe + Clone> {
-    /// Nothing measured
-    Off,
-    OneStatic(P),
-    OneDyn(Arc<dyn Probe>),
-    Two(P, Arc<dyn Probe>),
-}
-
-#[cfg(test)]
-impl<P: Probe + Clone> ProbeInternal<P> {
-    pub fn new(probe: P) -> Self {
-        Self::OneStatic(probe)
-    }
+    /// We have no internal [Probe] (from the ProbeFactory) but got a [Probe] via
+    /// the request API
+    RequestProbe(Arc<dyn Probe>),
+    /// We have a [Probe] from the internal probe factory
+    /// and got a [Probe] via
+    /// the request API
+    FactoryAndRequestProbe(P, Arc<dyn Probe>),
 }
 
 impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn effective_range(&self, range: InclusiveRange) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.effective_range(range),
-            ProbeInternal::OneStatic(p) => p.effective_range(range),
-            ProbeInternal::Two(p1, p2) => {
-                p1.effective_range(range);
-                p2.effective_range(range);
+            ProbeInternal::RequestProbe(p) => p.effective_range(range),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.effective_range(range);
+                request_probe.effective_range(range);
             }
         }
     }
@@ -196,12 +181,10 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn download_started(&self) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.download_started(),
-            ProbeInternal::OneStatic(p) => p.download_started(),
-            ProbeInternal::Two(p1, p2) => {
-                p1.download_started();
-                p2.download_started();
+            ProbeInternal::RequestProbe(p) => p.download_started(),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.download_started();
+                request_probe.download_started();
             }
         }
     }
@@ -209,12 +192,10 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn download_completed(&self, time: Duration) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.download_completed(time),
-            ProbeInternal::OneStatic(p) => p.download_completed(time),
-            ProbeInternal::Two(p1, p2) => {
-                p1.download_completed(time);
-                p2.download_completed(time);
+            ProbeInternal::RequestProbe(p) => p.download_completed(time),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.download_completed(time);
+                request_probe.download_completed(time);
             }
         }
     }
@@ -222,12 +203,10 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn download_failed(&self, time: Option<Duration>) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.download_failed(time),
-            ProbeInternal::OneStatic(p) => p.download_failed(time),
-            ProbeInternal::Two(p1, p2) => {
-                p1.download_failed(time);
-                p2.download_failed(time);
+            ProbeInternal::RequestProbe(p) => p.download_failed(time),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.download_failed(time);
+                request_probe.download_failed(time);
             }
         }
     }
@@ -235,12 +214,10 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn retry_attempt(&self, location: &dyn fmt::Display, error: &CondowError, next_in: Duration) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.retry_attempt(location, error, next_in),
-            ProbeInternal::OneStatic(p) => p.retry_attempt(location, error, next_in),
-            ProbeInternal::Two(p1, p2) => {
-                p1.retry_attempt(location, error, next_in);
-                p2.retry_attempt(location, error, next_in);
+            ProbeInternal::RequestProbe(p) => p.retry_attempt(location, error, next_in),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.retry_attempt(location, error, next_in);
+                request_probe.retry_attempt(location, error, next_in);
             }
         }
     }
@@ -254,16 +231,12 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
         remaining_range: InclusiveRange,
     ) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => {
+            ProbeInternal::RequestProbe(p) => {
                 p.stream_resume_attempt(location, error, orig_range, remaining_range)
             }
-            ProbeInternal::OneStatic(p) => {
-                p.stream_resume_attempt(location, error, orig_range, remaining_range)
-            }
-            ProbeInternal::Two(p1, p2) => {
-                p1.stream_resume_attempt(location, error, orig_range, remaining_range);
-                p2.stream_resume_attempt(location, error, orig_range, remaining_range);
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.stream_resume_attempt(location, error, orig_range, remaining_range);
+                request_probe.stream_resume_attempt(location, error, orig_range, remaining_range);
             }
         }
     }
@@ -271,12 +244,10 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn panic_detected(&self, msg: &str) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.panic_detected(msg),
-            ProbeInternal::OneStatic(p) => p.panic_detected(msg),
-            ProbeInternal::Two(p1, p2) => {
-                p1.panic_detected(msg);
-                p2.panic_detected(msg);
+            ProbeInternal::RequestProbe(p) => p.panic_detected(msg),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.panic_detected(msg);
+                request_probe.panic_detected(msg);
             }
         }
     }
@@ -284,33 +255,21 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn queue_full(&self) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.queue_full(),
-            ProbeInternal::OneStatic(p) => p.queue_full(),
-            ProbeInternal::Two(p1, p2) => {
-                p1.queue_full();
-                p2.queue_full();
+            ProbeInternal::RequestProbe(p) => p.queue_full(),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.queue_full();
+                request_probe.queue_full();
             }
         }
     }
 
     #[inline]
-    fn chunk_completed(
-        &self,
-        part_index: u64,
-        chunk_index: usize,
-        n_bytes: usize,
-        time: std::time::Duration,
-    ) {
+    fn chunk_received(&self, part_index: u64, chunk_index: usize, n_bytes: usize) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.chunk_completed(part_index, chunk_index, n_bytes, time),
-            ProbeInternal::OneStatic(p) => {
-                p.chunk_completed(part_index, chunk_index, n_bytes, time)
-            }
-            ProbeInternal::Two(p1, p2) => {
-                p1.chunk_completed(part_index, chunk_index, n_bytes, time);
-                p2.chunk_completed(part_index, chunk_index, n_bytes, time);
+            ProbeInternal::RequestProbe(p) => p.chunk_received(part_index, chunk_index, n_bytes),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.chunk_received(part_index, chunk_index, n_bytes);
+                request_probe.chunk_received(part_index, chunk_index, n_bytes);
             }
         }
     }
@@ -318,12 +277,10 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn part_started(&self, part_index: u64, range: crate::InclusiveRange) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.part_started(part_index, range),
-            ProbeInternal::OneStatic(p) => p.part_started(part_index, range),
-            ProbeInternal::Two(p1, p2) => {
-                p1.part_started(part_index, range);
-                p2.part_started(part_index, range);
+            ProbeInternal::RequestProbe(p) => p.part_started(part_index, range),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.part_started(part_index, range);
+                request_probe.part_started(part_index, range);
             }
         }
     }
@@ -337,12 +294,10 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
         time: std::time::Duration,
     ) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.part_completed(part_index, n_chunks, n_bytes, time),
-            ProbeInternal::OneStatic(p) => p.part_completed(part_index, n_chunks, n_bytes, time),
-            ProbeInternal::Two(p1, p2) => {
-                p1.part_completed(part_index, n_chunks, n_bytes, time);
-                p2.part_completed(part_index, n_chunks, n_bytes, time);
+            ProbeInternal::RequestProbe(p) => p.part_completed(part_index, n_chunks, n_bytes, time),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.part_completed(part_index, n_chunks, n_bytes, time);
+                request_probe.part_completed(part_index, n_chunks, n_bytes, time);
             }
         }
     }
@@ -350,43 +305,14 @@ impl<P: Probe + Clone> Probe for ProbeInternal<P> {
     #[inline]
     fn part_failed(&self, error: &CondowError, part_index: u64, range: &InclusiveRange) {
         match self {
-            ProbeInternal::Off => {}
-            ProbeInternal::OneDyn(p) => p.part_failed(error, part_index, range),
-            ProbeInternal::OneStatic(p) => p.part_failed(error, part_index, range),
-            ProbeInternal::Two(p1, p2) => {
-                p1.part_failed(error, part_index, range);
-                p2.part_failed(error, part_index, range);
+            ProbeInternal::RequestProbe(p) => p.part_failed(error, part_index, range),
+            ProbeInternal::FactoryAndRequestProbe(factory_probe, request_probe) => {
+                factory_probe.part_failed(error, part_index, range);
+                request_probe.part_failed(error, part_index, range);
             }
         }
     }
 }
 
-impl From<Option<Arc<dyn Probe>>> for ProbeInternal<()> {
-    fn from(probe: Option<Arc<dyn Probe>>) -> Self {
-        match probe {
-            Some(probe) => Self::OneDyn(probe),
-            None => Self::Off,
-        }
-    }
-}
-
-impl<P: Probe + Clone> From<Option<P>> for ProbeInternal<P>
-where
-    P: Probe + Clone,
-{
-    fn from(probe: Option<P>) -> Self {
-        match probe {
-            Some(probe) => Self::OneStatic(probe),
-            None => Self::Off,
-        }
-    }
-}
-
-impl Default for ProbeInternal<()> {
-    fn default() -> Self {
-        Self::Off
-    }
-}
-
 #[cfg(test)]
 mod tests;
