@@ -74,11 +74,10 @@
 //!
 //! // Methods of Probe have noop default implementations
 //! impl Probe for MyProbe {
-//!     fn chunk_completed(&self,
+//!     fn chunk_received(&self,
 //!         _part_index: u64,
 //!         _chunk_index: usize,
-//!         n_bytes: usize,
-//!         _time: Duration) {
+//!         n_bytes: usize) {
 //!         self.bytes_received.fetch_add(n_bytes, Ordering::SeqCst);
 //!     }
 //! }
@@ -138,25 +137,30 @@ pub trait Probe: Send + Sync + 'static {
     fn effective_range(&self, range: InclusiveRange) {}
 
     /// The actual IO started
+    #[inline]
     fn download_started(&self) {}
 
     /// IO tasks finished
     ///
     /// **This always is the last method called on a [Probe] if the download was successful.**
+    #[inline]
     fn download_completed(&self, time: Duration) {}
 
     /// IO tasks finished
     ///
     /// **This always is the last method called on a [Probe] if the download failed.**
+    #[inline]
     fn download_failed(&self, time: Option<Duration>) {}
 
     /// An error occurred but a retry will be attempted
+    #[inline]
     fn retry_attempt(&self, location: &dyn fmt::Display, error: &CondowError, next_in: Duration) {}
 
     /// A stream for fetching a part broke and an attempt to resume will be made
     ///
     /// `orig_range` is the original range for the download attempted.
     /// `remaining_range` the tail of the part which is still missing.
+    #[inline]
     fn stream_resume_attempt(
         &self,
         location: &dyn fmt::Display,
@@ -169,21 +173,37 @@ pub trait Probe: Send + Sync + 'static {
     /// A panic was detected
     ///
     /// Unless from a bug in this library it is most likely caused by the [crate::condow_client::CondowClient] implementation
+    #[inline]
     fn panic_detected(&self, msg: &str) {}
 
     /// All queues are full so no new request could be scheduled
+    #[inline]
     fn queue_full(&self) {}
 
     /// A part was completed
+    #[inline]
+    fn chunk_received(&self, part_index: u64, chunk_index: usize, n_bytes: usize) {
+        // Just for the time being as long as chunk_completed is not removed
+        #[allow(deprecated)]
+        self.chunk_completed(part_index, chunk_index, n_bytes, Duration::ZERO)
+    }
+
+    /// A part was completed
+    #[inline]
+    #[deprecated(note = "use chunk_received instead", since = "0.17.1")]
     fn chunk_completed(&self, part_index: u64, chunk_index: usize, n_bytes: usize, time: Duration) {
     }
+
     /// Download of a part has started
+    #[inline]
     fn part_started(&self, part_index: u64, range: InclusiveRange) {}
 
     /// Download of a part was completed
+    #[inline]
     fn part_completed(&self, part_index: u64, n_chunks: usize, n_bytes: u64, time: Duration) {}
 
     /// Download of a part failed
+    #[inline]
     fn part_failed(&self, error: &CondowError, part_index: u64, range: &InclusiveRange) {}
 }
 
@@ -220,14 +240,12 @@ mod simple_reporter {
     #[derive(Clone)]
     pub struct SimpleReporter {
         inner: Arc<Inner>,
-        skip_first_chunk_timings: bool,
     }
 
     impl SimpleReporter {
-        pub fn new(location: &dyn fmt::Display, skip_first_chunk_timings: bool) -> Self {
+        pub fn new(location: &dyn fmt::Display) -> Self {
             SimpleReporter {
                 inner: Arc::new(Inner::new(location.to_string())),
-                skip_first_chunk_timings,
             }
         }
 
@@ -292,7 +310,7 @@ mod simple_reporter {
 
     impl Default for SimpleReporter {
         fn default() -> Self {
-            Self::new(&"unknown".to_string(), false)
+            Self::new(&"unknown".to_string())
         }
     }
 
@@ -377,22 +395,11 @@ mod simple_reporter {
             self.inner.n_queue_full.fetch_add(1, Ordering::SeqCst);
         }
 
-        fn chunk_completed(
-            &self,
-            _part_index: u64,
-            chunk_index: usize,
-            n_bytes: usize,
-            time: Duration,
-        ) {
+        fn chunk_received(&self, _part_index: u64, _chunk_index: usize, n_bytes: usize) {
             let inner = self.inner.as_ref();
             inner.n_chunks_received.fetch_add(1, Ordering::SeqCst);
             inner.min_chunk_bytes.fetch_min(n_bytes, Ordering::SeqCst);
             inner.max_chunk_bytes.fetch_max(n_bytes, Ordering::SeqCst);
-            if self.skip_first_chunk_timings && chunk_index == 0 {
-                let us = time.as_micros() as u64;
-                inner.min_chunk_us.fetch_min(us, Ordering::SeqCst);
-                inner.max_chunk_us.fetch_max(us, Ordering::SeqCst);
-            }
         }
 
         fn part_completed(&self, _part_index: u64, n_chunks: usize, n_bytes: u64, time: Duration) {
