@@ -2,13 +2,12 @@ use std::{sync::Arc, time::Duration};
 
 use anyhow::{bail, Error as AnyError};
 use bytes::Bytes;
-use futures::{channel::mpsc, Stream, StreamExt};
+use futures::{channel::mpsc, Future, Stream, StreamExt};
 use tracing::{debug, debug_span, warn, Instrument, Span};
 
 use crate::{
     condow_client::{CondowClient, DownloadSpec},
     errors::{CondowError, IoError},
-    machinery::ProbeInternal,
     probe::Probe,
     streams::{BytesHint, BytesStream},
     InclusiveRange,
@@ -352,7 +351,7 @@ where
     pub async fn get_size<P: Probe + Clone>(
         &self,
         location: C::Location,
-        probe: &ProbeInternal<P>,
+        probe: &P,
     ) -> Result<u64, CondowError> {
         let parent = Span::current();
         let span = debug_span!(parent: &parent, "client_get_size");
@@ -371,27 +370,32 @@ where
         f.await
     }
 
-    pub async fn download<P: Probe + Clone>(
+    pub fn download<P: Probe + Clone>(
         &self,
         location: C::Location,
         spec: DownloadSpec,
-        probe: &ProbeInternal<P>,
-    ) -> Result<(BytesStream, BytesHint), CondowError> {
+        probe: P,
+    ) -> impl Future<Output = Result<(BytesStream, BytesHint), CondowError>> + Send + 'static {
         let parent = Span::current();
         let span = debug_span!(parent: &parent, "client_download_part", %spec);
         debug!(parent: &span, "downloading part");
 
-        let f = async {
-            let (client, config) = self.inner.as_ref();
+        let inner = Arc::clone(&self.inner);
+        async move {
+            let (client, config) = inner.as_ref();
             if let Some(config) = config {
                 retry_download(client, location, spec, config, probe.clone()).await
             } else {
                 Ok(client.download(location, spec).await?)
             }
         }
-        .instrument(span);
+        .instrument(span)
+    }
 
-        f.await
+    /// Returns the inner [CondowClient]
+    #[allow(dead_code)]
+    pub fn inner_client(&self) -> &C {
+        &self.inner.0
     }
 }
 
@@ -410,7 +414,7 @@ async fn retry_get_size<C, P: Probe + Clone>(
     client: &C,
     location: C::Location,
     config: &RetryConfig,
-    probe: &ProbeInternal<P>,
+    probe: &P,
 ) -> Result<u64, CondowError>
 where
     C: CondowClient,
@@ -449,7 +453,7 @@ async fn retry_download<C, P: Probe + Clone>(
     location: C::Location,
     spec: DownloadSpec,
     config: &RetryConfig,
-    probe: ProbeInternal<P>,
+    probe: P,
 ) -> Result<(BytesStream, BytesHint), CondowError>
 where
     C: CondowClient,
@@ -510,7 +514,7 @@ where
 struct RetryLoopPanicGuard<P: Probe + Clone> {
     completed_without_panic: bool,
     next_elem_tx: mpsc::UnboundedSender<Result<Bytes, IoError>>,
-    probe: ProbeInternal<P>,
+    probe: P,
 }
 
 impl<P: Probe + Clone> Drop for RetryLoopPanicGuard<P> {
@@ -535,7 +539,7 @@ async fn loop_retry_complete_stream<C, P: Probe + Clone>(
     client: C,
     next_elem_tx: mpsc::UnboundedSender<Result<Bytes, IoError>>,
     config: RetryConfig,
-    probe: ProbeInternal<P>,
+    probe: P,
 ) where
     C: CondowClient,
 {
@@ -643,7 +647,7 @@ async fn retry_download_get_stream<C, P: Probe + Clone>(
     location: C::Location,
     spec: DownloadSpec,
     config: &RetryConfig,
-    probe: &ProbeInternal<P>,
+    probe: &P,
 ) -> Result<(BytesStream, BytesHint), CondowError>
 where
     C: CondowClient,
