@@ -20,12 +20,17 @@ use crate::{
     DownloadRange, Downloads, Params, RequestNoLocation,
 };
 
-#[derive(Clone)]
+use self::stream_penderizer::{Penderizer, PenderizerModule};
+
+/// TODO: make members private
 pub struct TestCondowClient {
     pub data: Arc<Vec<u8>>,
     pub max_jitter_ms: usize,
     pub include_size_hint: bool,
     pub max_chunk_size: usize,
+    pub pending_on_stream_module: Option<PenderizerModule>,
+    pub pending_on_requests_module: Option<Mutex<PenderizerModule>>,
+    pub initial_pendings_on_request: usize,
 }
 
 impl TestCondowClient {
@@ -35,9 +40,24 @@ impl TestCondowClient {
             max_jitter_ms: 0,
             include_size_hint: true,
             max_chunk_size: 10,
+            pending_on_stream_module: None,
+            pending_on_requests_module: None,
+            initial_pendings_on_request: 0,
         }
     }
 
+    pub fn pending_on_stream_n_times(mut self, consecutive_pendings: usize) -> Self {
+        self.pending_on_stream_module = Some(PenderizerModule::new(consecutive_pendings));
+        self
+    }
+
+    pub fn pending_on_request_n_times(mut self, consecutive_pendings: usize) -> Self {
+        self.initial_pendings_on_request = consecutive_pendings;
+        self.pending_on_requests_module =
+            Some(Mutex::new(PenderizerModule::new(consecutive_pendings)));
+        self
+    }
+
     pub fn max_jitter_ms(mut self, max_jitter_ms: usize) -> Self {
         self.max_jitter_ms = max_jitter_ms;
         self
@@ -56,6 +76,31 @@ impl TestCondowClient {
     pub fn data(&self) -> Arc<Vec<u8>> {
         Arc::clone(&self.data)
     }
+
+    pub fn data_slice(&self) -> &[u8] {
+        &self.data
+    }
+}
+
+impl Clone for TestCondowClient {
+    fn clone(&self) -> Self {
+        let pending_on_requests_module = if self.initial_pendings_on_request > 0 {
+            Some(Mutex::new(PenderizerModule::new(
+                self.initial_pendings_on_request,
+            )))
+        } else {
+            None
+        };
+        Self {
+            data: Arc::clone(&self.data),
+            max_jitter_ms: self.max_jitter_ms,
+            include_size_hint: self.include_size_hint,
+            max_chunk_size: self.max_chunk_size,
+            pending_on_stream_module: self.pending_on_stream_module,
+            pending_on_requests_module,
+            initial_pendings_on_request: self.initial_pendings_on_request,
+        }
+    }
 }
 
 impl Default for TestCondowClient {
@@ -86,51 +131,69 @@ impl CondowClient for TestCondowClient {
             crate::errors::CondowError,
         >,
     > {
-        let range = match spec {
-            DownloadSpec::Complete => 0..self.data.len(),
-            DownloadSpec::Range(r) => {
-                let r = r.to_std_range_excl();
-                r.start as usize..r.end as usize
-            }
-        };
-
-        if range.end > self.data.len() {
-            return Box::pin(future::ready(Err(CondowError::new_invalid_range(format!(
-                "max upper bound is {} but {} was requested",
-                self.data.len() - 1,
-                range.end - 1
-            )))));
-        }
-
-        let slice = &self.data[range];
-
-        let bytes_hint = if self.include_size_hint {
-            BytesHint::new_exact(slice.len() as u64)
+        let should_be_pending = if let Some(module) = &self.pending_on_requests_module {
+            module.lock().unwrap().return_pending()
         } else {
-            BytesHint::new_no_hint()
+            false
         };
 
-        let iter = slice
-            .chunks(self.max_chunk_size)
-            .map(Bytes::copy_from_slice)
-            .map(Ok);
+        let me = self.clone();
 
-        let owned_bytes: Vec<_> = iter.collect();
-
-        let jitter = self.max_jitter_ms;
-        let stream = stream::iter(owned_bytes).then(move |bytes| async move {
-            if jitter > 0 {
-                let jitter = OsRng.gen_range(0..=(jitter as u64));
-                time::sleep(Duration::from_millis(jitter)).await;
+        async move {
+            if should_be_pending {
+                tokio::task::yield_now().await;
             }
-            bytes
-        });
 
-        let stream: BytesStream = Box::pin(stream);
+            let range = match spec {
+                DownloadSpec::Complete => 0..me.data.len(),
+                DownloadSpec::Range(r) => {
+                    let r = r.to_std_range_excl();
+                    r.start as usize..r.end as usize
+                }
+            };
 
-        let f = future::ready(Ok((stream, bytes_hint)));
+            if range.end > me.data.len() {
+                return Err(CondowError::new_invalid_range(format!(
+                    "max upper bound is {} but {} was requested",
+                    me.data.len() - 1,
+                    range.end - 1
+                )));
+            }
 
-        Box::pin(f)
+            let slice = &me.data[range];
+
+            let bytes_hint = if me.include_size_hint {
+                BytesHint::new_exact(slice.len() as u64)
+            } else {
+                BytesHint::new_no_hint()
+            };
+
+            let iter = slice
+                .chunks(me.max_chunk_size)
+                .map(Bytes::copy_from_slice)
+                .map(Ok);
+
+            let owned_bytes: Vec<_> = iter.collect();
+
+            let jitter = me.max_jitter_ms;
+            let stream = stream::iter(owned_bytes).then(move |bytes| async move {
+                if jitter > 0 {
+                    let jitter = OsRng.gen_range(0..=(jitter as u64));
+                    time::sleep(Duration::from_millis(jitter)).await;
+                }
+                bytes
+            });
+
+            let stream: BytesStream = if let Some(pending_module) = me.pending_on_stream_module {
+                let stream_with_pending = Penderizer::new(stream, pending_module.clone());
+                stream_with_pending.boxed()
+            } else {
+                stream.boxed()
+            };
+
+            Ok((stream, bytes_hint))
+        }
+        .boxed()
     }
 }
 
@@ -144,6 +207,26 @@ pub fn create_test_data() -> Vec<u8> {
     data
 }
 
+#[tokio::test]
+async fn test_test_client() {
+    use futures::TryStreamExt;
+    let client = TestCondowClient::new().max_jitter_ms(5);
+
+    let (bytes_stream, _bytes_hint) = client
+        .download(IgnoreLocation, (10..=30).into())
+        .await
+        .unwrap();
+    let result = bytes_stream
+        .try_fold(Vec::new(), |mut acc, chunk| {
+            acc.extend_from_slice(&chunk);
+            future::ok(acc)
+        })
+        .await
+        .unwrap();
+
+    assert_eq!(result, client.data_slice()[10usize..=30]);
+}
+
 pub fn create_chunk_stream(
     n_parts: u64,
     n_chunks: usize,
@@ -216,14 +299,14 @@ pub fn create_chunk_stream(
 
     parts.shuffle(&mut rng);
 
-    let (chunk_stream, tx) = ChunkStream::new(bytes_hint);
+    let (chunk_stream, tx) = ChunkStream::new_channel_sink_pair(bytes_hint);
 
     loop {
         let n = rng.gen_range(0..parts.len());
 
         let part = &mut parts[n];
         let chunk = part.remove(0);
-        let _ = tx.unbounded_send(chunk);
+        let _ = tx.send(chunk);
         if part.is_empty() {
             parts.remove(n);
         }
@@ -311,12 +394,12 @@ pub fn create_chunk_stream_with_err(
 
     parts.shuffle(&mut rng);
 
-    let (chunk_stream, tx) = ChunkStream::new(bytes_hint);
+    let (chunk_stream, tx) = ChunkStream::new_channel_sink_pair(bytes_hint);
 
     let mut err_sent = chunks_left_until_err == 0;
     loop {
         if chunks_left_until_err == 0 {
-            let _ = tx.unbounded_send(Err(CondowError::new_other("forced stream error")));
+            let _ = tx.send(Err(CondowError::new_other("forced stream error")));
             err_sent = true;
             break;
         }
@@ -325,7 +408,7 @@ pub fn create_chunk_stream_with_err(
 
         let part = &mut parts[n];
         let chunk = part.remove(0);
-        let _ = tx.unbounded_send(chunk);
+        let _ = tx.send(chunk);
         if part.is_empty() {
             parts.remove(n);
         }
@@ -336,7 +419,7 @@ pub fn create_chunk_stream_with_err(
     }
 
     if !err_sent {
-        let _ = tx.unbounded_send(Err(CondowError::new_other("forced stream error after end")));
+        let _ = tx.send(Err(CondowError::new_other("forced stream error after end")));
     }
 
     (chunk_stream, values)
@@ -489,7 +572,8 @@ fn make_a_stream(
 
     let mut chunk_patterns = pattern.into_iter().cycle().enumerate();
 
-    let (chunk_stream, tx) = ChunkStream::new(BytesHint::new_exact(bytes.len() as u64));
+    let (chunk_stream, tx) =
+        ChunkStream::new_channel_sink_pair(BytesHint::new_exact(bytes.len() as u64));
 
     tokio::spawn(async move {
         let mut start = 0;
@@ -503,7 +587,7 @@ fn make_a_stream(
                     if let Some(pattern) = pattern {
                         (chunk_index, pattern)
                     } else {
-                        let _ = tx.unbounded_send(Err(CondowError::new_other("test error")));
+                        let _ = tx.send(Err(CondowError::new_other("test error")));
                         break;
                     }
                 } else {
@@ -523,7 +607,7 @@ fn make_a_stream(
                 bytes_left,
             };
 
-            let _ = tx.unbounded_send(Ok(chunk));
+            let _ = tx.send(Ok(chunk));
 
             start = end_excl;
         }
@@ -532,6 +616,106 @@ fn make_a_stream(
     Ok(chunk_stream)
 }
 
+pub mod stream_penderizer {
+    //! Makes a stream more pending...
+
+    use std::{fmt, task::Poll};
+
+    use futures::Stream;
+    use pin_project_lite::pin_project;
+
+    pin_project! {
+        /// Adds pendings to poll
+        ///
+        /// Useful to test for codepaths on pending in
+        /// custom stream implementations.
+        ///
+        ///
+        /// TODO: Can something be named like this?
+        pub struct Penderizer<St> {
+            #[pin]
+            incoming: St,
+            pending_module: PenderizerModule,
+        }
+    }
+
+    impl<St> Penderizer<St>
+    where
+        St: Stream + Send + 'static,
+    {
+        pub fn new(incoming: St, pending_module: PenderizerModule) -> Self {
+            Self {
+                incoming,
+                pending_module,
+            }
+        }
+    }
+
+    impl<St> Stream for Penderizer<St>
+    where
+        St: Stream + Send + 'static,
+    {
+        type Item = St::Item;
+
+        fn poll_next(
+            self: std::pin::Pin<&mut Self>,
+            cx: &mut std::task::Context<'_>,
+        ) -> std::task::Poll<Option<Self::Item>> {
+            let this = self.project();
+
+            if this.pending_module.return_pending() {
+                cx.waker().wake_by_ref();
+                return Poll::Pending;
+            }
+
+            match this.incoming.poll_next(cx) {
+                Poll::Ready(Some(item)) => Poll::Ready(Some(item)),
+                Poll::Ready(None) => Poll::Ready(None),
+                Poll::Pending => {
+                    panic!("input must never return pending")
+                }
+            }
+        }
+    }
+
+    #[derive(Clone, Copy)]
+    pub struct PenderizerModule {
+        /// Zero means no pendings are added
+        consecutive_pendings: usize,
+        pendings_done: usize,
+    }
+
+    impl PenderizerModule {
+        pub fn new(consecutive_pendings: usize) -> Self {
+            Self {
+                consecutive_pendings,
+                pendings_done: 0,
+            }
+        }
+
+        pub fn return_pending(&mut self) -> bool {
+            let pending = if self.pendings_done < self.consecutive_pendings {
+                self.pendings_done += 1;
+                true
+            } else {
+                self.pendings_done = 0;
+                false
+            };
+            pending
+        }
+    }
+
+    impl fmt::Display for PenderizerModule {
+        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+            write!(
+                f,
+                "Consecutive pendings: {}, pendings done: {}",
+                self.consecutive_pendings, self.pendings_done
+            )
+        }
+    }
+}
+
 #[tokio::test]
 async fn check_test_downloader_1() {
     // Use the range patterns completely and even cycle them
